@@ -32,6 +32,8 @@
 #include <libgnomeui/libgnomeui.h>
 #include <libbonoboui.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
+
+#include "async-pixbuf-ops.h"
 #include "catalog.h"
 #include "comments.h"
 #include "dlg-bookmarks.h"
@@ -57,6 +59,7 @@
 #include "dlg-search.h"
 #include "fullscreen.h"
 #include "gconf-utils.h"
+#include "gth-pixbuf-op.h"
 #include "gthumb-error.h"
 #include "gthumb-window.h"
 #include "gtk-utils.h"
@@ -161,7 +164,21 @@ file_save_command_impl (BonoboUIComponent *uic,
 			const gchar       *verbname)
 {
 	GThumbWindow *window = user_data;
-	dlg_save_image (window, image_viewer_get_current_pixbuf (IMAGE_VIEWER (window->viewer)));
+	char         *current_folder = NULL;
+
+	if (window->image_path != NULL)
+		current_folder = g_strdup (window->image_path);
+
+	else if (window->dir_list->path != NULL)
+		current_folder = g_strconcat (window->dir_list->path,
+					      "/", 
+					      NULL);
+
+	dlg_save_image (GTK_WINDOW (window->app), 
+			current_folder,
+			image_viewer_get_current_pixbuf (IMAGE_VIEWER (window->viewer)));
+
+	g_free (current_folder);
 }
 
 
@@ -211,7 +228,7 @@ rename_file (GThumbWindow *window,
 	     const GList  *list)
 {
 	const char   *old_name, *old_path;
-	char         *old_name_utf8;
+	char         *old_name_utf8, *new_name_utf8;
 	char         *new_name, *new_path;
 	char         *dir;
 
@@ -221,18 +238,21 @@ rename_file (GThumbWindow *window,
 	old_name = file_name_from_path (old_path);
 	old_name_utf8 = g_locale_to_utf8 (old_name, -1, NULL, NULL, NULL);
 
-	new_name = _gtk_request_dialog_run (GTK_WINDOW (window->app),
-					    GTK_DIALOG_MODAL,
-					    _("Enter the new name: "),
-					    old_name_utf8,
-					    MAX_NAME_LEN,
-					    GTK_STOCK_CANCEL,
-					    _("_Rename"));
+	new_name_utf8 = _gtk_request_dialog_run (GTK_WINDOW (window->app),
+						 GTK_DIALOG_MODAL,
+						 _("Enter the new name: "),
+						 old_name_utf8,
+						 MAX_NAME_LEN,
+						 GTK_STOCK_CANCEL,
+						 _("_Rename"));
 	g_free (old_name_utf8);
 
-	if (new_name == NULL) 
+	if (new_name_utf8 == NULL) 
 		return;
 
+	new_name = g_locale_from_utf8 (new_name_utf8, -1, 0, 0, 0);
+	g_free (new_name_utf8);
+
 	if (strchr (new_name, '/') != NULL) {
 		char *utf8_name;
 
@@ -348,17 +368,16 @@ image_rename_command_impl (BonoboUIComponent *uic,
 
 static void 
 duplicate_file (GThumbWindow *window,
-		const GList  *list)
+		const char   *old_path)
 {
-	const char   *old_name, *old_path, *ext;
-	char         *old_name_no_ext;
-	char         *new_name, *new_path;
-	char         *dir;
-	int           try;
+	const char *old_name, *ext;
+	char       *old_name_no_ext;
+	char       *new_name, *new_path;
+	char       *dir;
+	int         try;
 
-	g_return_if_fail (list != NULL);
+	g_return_if_fail (old_path != NULL);
 		
-	old_path = list->data;
 	old_name = file_name_from_path (old_path);
 	old_name_no_ext = remove_extension_from_path (old_name);
 	ext = strrchr (old_name, '.');
@@ -384,6 +403,7 @@ duplicate_file (GThumbWindow *window,
 	if (file_copy (old_path, new_path)) {
 		cache_copy (old_path, new_path);
 		comment_copy (old_path, new_path);
+
 	} else {
 		char *utf8_path;
 
@@ -400,6 +420,19 @@ duplicate_file (GThumbWindow *window,
 }
 
 
+static void 
+duplicate_file_list (GThumbWindow *window,
+		     const GList  *list)
+{
+	const GList *scan;
+
+	for (scan = list; scan; scan = scan->next) {
+		char *filename = scan->data;
+		duplicate_file (window, filename);
+	}
+}
+
+
 void 
 edit_duplicate_file_command_impl (BonoboUIComponent *uic, 
 				  gpointer           user_data, 
@@ -411,7 +444,7 @@ edit_duplicate_file_command_impl (BonoboUIComponent *uic,
 	list = ilist_utils_get_file_list_selection (IMAGE_LIST (window->file_list->ilist));
 	g_return_if_fail (list != NULL);
 
-	duplicate_file (window, list);
+	duplicate_file_list (window, list);
 	path_list_free (list);
 }
 
@@ -428,7 +461,7 @@ image_duplicate_command_impl (BonoboUIComponent *uic,
 		return;
 
 	list = g_list_prepend (NULL, g_strdup (window->image_path));
-	duplicate_file (window, list);
+	duplicate_file_list (window, list);
 	path_list_free (list);
 }
 
@@ -576,9 +609,9 @@ edit_current_edit_comment_command_impl (BonoboUIComponent *uic,
 	if (window->image_path == NULL)
 		return;
 
-	pos = file_list_pos_from_path (window->file_list, window->image_path);
+	pos = gth_file_list_pos_from_path (window->file_list, window->image_path);
 	if (pos != -1) {
-		file_list_select_image_by_pos (window->file_list, pos);
+		gth_file_list_select_image_by_pos (window->file_list, pos);
 		dlg_edit_comment (NULL, window);
 	}
 }
@@ -605,6 +638,7 @@ edit_delete_comment_command_impl (BonoboUIComponent *uic,
 			comments_save_comment (filename, cdata);
 		else
 			comment_delete (filename);
+		comment_data_free (cdata);
 
 		all_windows_notify_update_comment (filename);
 	}
@@ -633,9 +667,9 @@ edit_current_edit_categories_command_impl (BonoboUIComponent *uic,
 	if (window->image_path == NULL)
 		return;
 
-	pos = file_list_pos_from_path (window->file_list, window->image_path);
+	pos = gth_file_list_pos_from_path (window->file_list, window->image_path);
 	if (pos != -1) {
-		file_list_select_image_by_pos (window->file_list, pos);
+		gth_file_list_select_image_by_pos (window->file_list, pos);
 		dlg_categories (NULL, window);
 	}
 }
@@ -833,7 +867,7 @@ create_new_folder_or_library (GThumbWindow *window,
 			      char         *str_error)
 {
 	const char   *current_path;
-	char         *new_name;
+	char         *new_name, *new_name_utf8;
 	char         *new_path;
 
 	if (window->sidebar_content == DIR_LIST)
@@ -843,17 +877,20 @@ create_new_folder_or_library (GThumbWindow *window,
 	else
 		return;
 
-	new_name = _gtk_request_dialog_run (GTK_WINDOW (window->app),
-					    GTK_DIALOG_MODAL,
-					    str_prompt,
-					    str_old_name,
-					    MAX_NAME_LEN,
-					    GTK_STOCK_CANCEL,
-					    _("C_reate"));
+	new_name_utf8 = _gtk_request_dialog_run (GTK_WINDOW (window->app),
+						 GTK_DIALOG_MODAL,
+						 str_prompt,
+						 str_old_name,
+						 MAX_NAME_LEN,
+						 GTK_STOCK_CANCEL,
+						 _("C_reate"));
 	
-	if (new_name == NULL) 
+	if (new_name_utf8 == NULL) 
 		return;
 
+	new_name = g_locale_from_utf8 (new_name_utf8, -1, 0, 0, 0);
+	g_free (new_name_utf8);
+
 	if (strchr (new_name, '/') != NULL) {
 		char *utf8_name;
 
@@ -944,24 +981,28 @@ folder_rename (GThumbWindow *window,
 	const char   *old_name;
 	char         *old_name_utf8;
 	char         *new_name;
+	char         *new_name_utf8;
 	char         *new_path;
 	char         *parent_path;
 
 	old_name = file_name_from_path (old_path);
 	old_name_utf8 = g_locale_to_utf8 (old_name, -1, NULL, NULL, NULL);
 
-	new_name = _gtk_request_dialog_run (GTK_WINDOW (window->app),
-					    GTK_DIALOG_MODAL,
-					    _("Enter the new name: "),
-					    old_name_utf8,
-					    MAX_NAME_LEN,
-					    GTK_STOCK_CANCEL,
-					    _("_Rename"));
+	new_name_utf8 = _gtk_request_dialog_run (GTK_WINDOW (window->app),
+						 GTK_DIALOG_MODAL,
+						 _("Enter the new name: "),
+						 old_name_utf8,
+						 MAX_NAME_LEN,
+						 GTK_STOCK_CANCEL,
+						 _("_Rename"));
 	g_free (old_name_utf8);
 	
-	if (new_name == NULL) 
+	if (new_name_utf8 == NULL) 
 		return;
 
+	new_name = g_locale_from_utf8 (new_name_utf8, -1, 0, 0, 0);
+	g_free (new_name_utf8);
+
 	if (strchr (new_name, '/') != NULL) {
 		char *utf8_name;
 
@@ -1324,6 +1365,7 @@ folder_copy__response_cb (GObject *object,
 				 new_path, 
 				 move, 
 				 TRUE,
+				 FALSE,
 				 folder_copy__continue, 
 				 g_strdup (old_path));
 
@@ -1513,6 +1555,7 @@ catalog_rename (GThumbWindow *window,
 	char         *name_only;
 	char         *name_only_utf8;
 	char         *new_name;
+	char         *new_name_utf8;
 	char         *new_catalog_path;
 	char         *path_only;
 	gboolean      is_dir;
@@ -1525,20 +1568,23 @@ catalog_rename (GThumbWindow *window,
 		name_only = g_strdup (file_name_from_path (catalog_path));
 	name_only_utf8 = g_locale_to_utf8 (name_only, -1, NULL, NULL, NULL);
 
-	new_name = _gtk_request_dialog_run (GTK_WINDOW (window->app),
-					    GTK_DIALOG_MODAL,
-					    _("Enter the new name: "),
-					    name_only_utf8,
-					    MAX_NAME_LEN,
-					    GTK_STOCK_CANCEL,
-					    _("_Rename"));
+	new_name_utf8 = _gtk_request_dialog_run (GTK_WINDOW (window->app),
+						 GTK_DIALOG_MODAL,
+						 _("Enter the new name: "),
+						 name_only_utf8,
+						 MAX_NAME_LEN,
+						 GTK_STOCK_CANCEL,
+						 _("_Rename"));
 	g_free (name_only_utf8);
 	
-	if (new_name == NULL) {
+	if (new_name_utf8 == NULL) {
 		g_free (name_only);
 		return;
 	}
 
+	new_name = g_locale_from_utf8 (new_name_utf8, -1, 0, 0, 0);
+	g_free (new_name_utf8);
+
 	if (strchr (new_name, '/') != NULL) {
 		char *utf8_name;
 
@@ -1614,22 +1660,26 @@ edit_current_catalog_new_command_impl (BonoboUIComponent *uic,
 {
 	GThumbWindow *window = user_data;
 	char         *new_name;
+	char         *new_name_utf8;
 	char         *new_catalog_path;
 	int           fd;
 
 	if (window->catalog_list->path == NULL)
 		return;
 
-	new_name = _gtk_request_dialog_run (GTK_WINDOW (window->app),
-					    GTK_DIALOG_MODAL,
-					    _("Enter the catalog name: "),
-					    _("New Catalog"),
-					    MAX_NAME_LEN,
-					    GTK_STOCK_CANCEL,
-					    GTK_STOCK_OK);
-	if (new_name == NULL) 
+	new_name_utf8 = _gtk_request_dialog_run (GTK_WINDOW (window->app),
+						 GTK_DIALOG_MODAL,
+						 _("Enter the catalog name: "),
+						 _("New Catalog"),
+						 MAX_NAME_LEN,
+						 GTK_STOCK_CANCEL,
+						 GTK_STOCK_OK);
+	if (new_name_utf8 == NULL) 
 		return;
 
+	new_name = g_locale_from_utf8 (new_name_utf8, -1, 0, 0, 0);
+	g_free (new_name_utf8);
+
 	if (strchr (new_name, '/') != NULL) {
 		char *utf8_name;
 
@@ -1961,14 +2011,15 @@ alter_image_desaturate_command_impl (BonoboUIComponent *uic,
 	ImageViewer  *viewer = IMAGE_VIEWER (window->viewer);
 	GdkPixbuf    *src_pixbuf;
 	GdkPixbuf    *dest_pixbuf;
+	GthPixbufOp  *pixop;
 	
 	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
 	dest_pixbuf = gdk_pixbuf_copy (src_pixbuf);
-	_gdk_pixbuf_desaturate (dest_pixbuf, dest_pixbuf);
-	image_viewer_set_pixbuf (viewer, dest_pixbuf);
+	pixop = _gdk_pixbuf_desaturate (dest_pixbuf, dest_pixbuf);
 	g_object_unref (dest_pixbuf);
 
-	window_image_modified (window, TRUE);
+	window_exec_pixbuf_op (window, pixop);
+	g_object_unref (pixop);
 }
 
 
@@ -1981,14 +2032,15 @@ alter_image_invert_command_impl (BonoboUIComponent *uic,
 	ImageViewer  *viewer = IMAGE_VIEWER (window->viewer);
 	GdkPixbuf    *src_pixbuf;
 	GdkPixbuf    *dest_pixbuf;
+	GthPixbufOp  *pixop;
 
 	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
 	dest_pixbuf = gdk_pixbuf_copy (src_pixbuf);
-	_gdk_pixbuf_invert (dest_pixbuf, dest_pixbuf);
-	image_viewer_set_pixbuf (viewer, dest_pixbuf);
+	pixop = _gdk_pixbuf_invert (dest_pixbuf, dest_pixbuf);
 	g_object_unref (dest_pixbuf);
 
-	window_image_modified (window, TRUE);
+	window_exec_pixbuf_op (window, pixop);
+	g_object_unref (pixop);
 }
 
 
@@ -2001,14 +2053,15 @@ alter_image_equalize_command_impl (BonoboUIComponent *uic,
 	ImageViewer  *viewer = IMAGE_VIEWER (window->viewer);
 	GdkPixbuf    *src_pixbuf;
 	GdkPixbuf    *dest_pixbuf;
+	GthPixbufOp  *pixop;
 
 	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
 	dest_pixbuf = gdk_pixbuf_copy (src_pixbuf);
-	_gdk_pixbuf_eq_histogram (dest_pixbuf, dest_pixbuf);
-	image_viewer_set_pixbuf (viewer, dest_pixbuf);
+	pixop = _gdk_pixbuf_eq_histogram (dest_pixbuf, dest_pixbuf);
 	g_object_unref (dest_pixbuf);
 
-	window_image_modified (window, TRUE);
+	window_exec_pixbuf_op (window, pixop);
+	g_object_unref (pixop);
 }
 
 
@@ -2021,14 +2074,15 @@ alter_image_adjust_levels_command_impl (BonoboUIComponent *uic,
 	ImageViewer  *viewer = IMAGE_VIEWER (window->viewer);
 	GdkPixbuf    *src_pixbuf;
 	GdkPixbuf    *dest_pixbuf;
+	GthPixbufOp  *pixop;
 
 	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
 	dest_pixbuf = gdk_pixbuf_copy (src_pixbuf);
-	_gdk_pixbuf_adjust_levels (dest_pixbuf, dest_pixbuf);
-	image_viewer_set_pixbuf (viewer, dest_pixbuf);
+	pixop = _gdk_pixbuf_adjust_levels (dest_pixbuf, dest_pixbuf);
 	g_object_unref (dest_pixbuf);
 
-	window_image_modified (window, TRUE);
+	window_exec_pixbuf_op (window, pixop);
+	g_object_unref (pixop);
 }
 
 
@@ -2688,7 +2742,7 @@ help_about_command_impl (BonoboUIComponent *uic,
 	logo = gdk_pixbuf_new_from_file (PIXMAPSDIR "/gthumb.png", NULL);
 	about = gnome_about_new (_("gThumb"), 
 				 VERSION,
-				 "Copyright (C) 2001 The Free Software Foundation, Inc.",
+				 _("Copyright (C) 2001 Free Software Foundation, Inc."),
 				 _("An image viewer and browser for GNOME."),
 				 authors,
 				 documenters,
